﻿// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.
// See the LICENSE file in the project root for more information.

using System.Threading;
using Microsoft.CodeAnalysis.CSharp.Syntax;
using Microsoft.CodeAnalysis.Shared.Collections;
using Microsoft.CodeAnalysis.Structure;

namespace Microsoft.CodeAnalysis.CSharp.Structure;

internal class MethodDeclarationStructureProvider : AbstractSyntaxNodeStructureProvider<MethodDeclarationSyntax>
{
    protected override void CollectBlockSpans(
        SyntaxToken previousToken,
        MethodDeclarationSyntax methodDeclaration,
        ref TemporaryArray<BlockSpan> spans,
        BlockStructureOptions options,
        CancellationToken cancellationToken)
    {
        CSharpStructureHelpers.CollectCommentBlockSpans(methodDeclaration, ref spans, options);

        // fault tolerance
        if (methodDeclaration.Body == null ||
            methodDeclaration.Body.OpenBraceToken.IsMissing ||
            methodDeclaration.Body.CloseBraceToken.IsMissing)
        {
            return;
        }

        SyntaxNodeOrToken current = methodDeclaration;
        var nextSibling = current.GetNextSibling();

        // Check IsNode to compress blank lines after this node if it is the last child of the parent.
        //
        // Whitespace between methods is collapsed in Metadata as Source.
        var compressEmptyLines = options.IsMetadataAsSource
            && (!nextSibling.IsNode || nextSibling.IsKind(SyntaxKind.MethodDeclaration));

        spans.AddIfNotNull(CSharpStructureHelpers.CreateBlockSpan(
            methodDeclaration,
            methodDeclaration.ParameterList.GetLastToken(includeZeroWidth: true),
            compressEmptyLines: compressEmptyLines,
            autoCollapse: true,
            type: BlockTypes.Member,
            isCollapsible: true));
    }
}
